@@ -25,6 +25,16 @@ class BasisFunction {
         return num_bfs;
     };
 
+    /* For preparing a vector of basis functions that maps a decoration number
+    to the old and new basis functions simultaneously.
+    This vector contains copies of the underlying basis functions.
+    The pair<double, double> is constructed as follows:
+    first -> Basis function of the *new* symbol
+    second -> Basis function of the *old* symbol
+    */
+    std::vector<std::pair<double, double>> prepare_bfs_new_old(unsigned int new_id,
+                                                               unsigned int old_id) const;
+
     /** Stream operator */
     friend std::ostream &operator<<(std::ostream &out, const BasisFunction &bf);
 
