@@ -1,9 +1,9 @@
 #include "symbols_with_numbers.hpp"
+
 #include <cstring>
 #include <iostream>
 
-Symbols::Symbols(const vec_str_t &symbs, const set_str_t &unique_symbs) : symbols(symbs)
-{
+Symbols::Symbols(const vec_str_t &symbs, const set_str_t &unique_symbs) : symbols(symbs) {
     symb_ids.resize(symbs.size());
 
 #ifdef PRINT_DEBUG
@@ -11,8 +11,7 @@ Symbols::Symbols(const vec_str_t &symbs, const set_str_t &unique_symbs) : symbol
 #endif
 
     unsigned int current_id = 0;
-    for (auto iter = unique_symbs.begin(); iter != unique_symbs.end(); ++iter)
-    {
+    for (auto iter = unique_symbs.begin(); iter != unique_symbs.end(); ++iter) {
         symb_id_translation[*iter] = current_id++;
     }
 
@@ -20,52 +19,42 @@ Symbols::Symbols(const vec_str_t &symbs, const set_str_t &unique_symbs) : symbol
     update_ids();
 }
 
-Symbols::Symbols(const Symbols &other)
-{
+Symbols::Symbols(const Symbols &other) {
     other.swap(*this);
 }
 
-Symbols &Symbols::operator=(const Symbols &other)
-{
+Symbols &Symbols::operator=(const Symbols &other) {
     other.swap(*this);
     return *this;
 }
 
-bool Symbols::is_consistent() const
-{
-    for (unsigned int i = 0; i < symbols.size(); i++)
-    {
-        if (symb_ids.at(i) != symb_id_translation.at(symbols[i]))
-        {
+bool Symbols::is_consistent() const {
+    for (unsigned int i = 0; i < symbols.size(); i++) {
+        if (symb_ids.at(i) != symb_id_translation.at(symbols[i])) {
             return false;
         }
     }
     return true;
 }
 
-void Symbols::set_symbol(unsigned int indx, const std::string &new_symb)
-{
+void Symbols::set_symbol(unsigned int indx, const std::string &new_symb) {
     symbols[indx] = new_symb;
     symb_ids[indx] = symb_id_translation[new_symb];
 }
 
-void Symbols::set_symbols(const vec_str_t &new_symbs)
-{
+void Symbols::set_symbols(const vec_str_t &new_symbs) {
     symb_ids.resize(new_symbs.size());
     symbols = new_symbs;
     update_ids();
 }
 
-void Symbols::update_ids()
-{
-    for (unsigned int i = 0; i < symbols.size(); i++)
-    {
+void Symbols::update_ids() {
+    for (unsigned int i = 0; i < symbols.size(); i++) {
         symb_ids.at(i) = symb_id_translation[symbols[i]];
     }
 }
 
-void Symbols::swap(Symbols &other) const
-{
+void Symbols::swap(Symbols &other) const {
     other.symbols = symbols;
     other.symb_id_translation = symb_id_translation;
     other.symb_ids = symb_ids;
